# Radiance Image support for PIL
#
# History:
#		99-21-08 gm		Created
#
# Copyright (C) Georg Mischler 1999.
#


__version__ = "0.1"

import string

import Image
import ImageFile

_radiance_format = 'radiance'

def _accept(prefix):
	return 1 # there are some older images without the header :(
	res = prefix[:10] == '#?RADIANCE'
	return res

def _save(im, fp, filename):
	print im.format
	if im.mode != 'RGBf':
		raise ValueError, 'Not a RGBf image'
	fp.write('#?RADIANCE\n')
	for cmd in im.info['commands']:
		fp.write(cmd + '\n')
	fp.write('*Processed by PIL extension*\n')
	if im.info['software']:
		# XXX We might add something here...
		fp.write('SOFTWARE=%s\n' % im.info['software'])
	if im.size[0] <= 0x7fff: # encoding limit
		fp.write('FORMAT=32-bit_rle_rgbe\n')
	if im.info['view']:
		fp.write(view + '\n')
	if im.info['exposure']:
		fp.write(exposure + '\n')
	for something in im.info['unknown']:
		fp.write(something + '\n')
	fp.write('\n-Y %d +X %d\n' % (im.size[1], im.size[0]))
	ImageFile._save(im, fp, [(
				'radiance', 
				(0, 0) + im.size,
				0,
				im.size, # we need the xsize to allocate an RGBE buffer
				)])

class RadianceImageFile(ImageFile.ImageFile):

	format = _radiance_format
	format_description = "Radiance Image"

	def _open(self):
		headerlines = self._readheaders(self.fp)
		self.info = self._parseheaders(headerlines)
		self.xsize = self._readimgsize(self.fp)
		self.size = (abs(self.xsize[0]), abs(self.xsize[1]))
		self.tile = [(
				_radiance_format,
				(0, 0) + self.size,
				self.start,
				self.size, # we need the xsize to allocate an RGBE buffer
				)]
		self.mode = 'RGBf'

	def _readheaders(self, file):
		pos = 0
		lines = []
		line = file.readline()
		while line and line != '\n':
			lines.append(line)
			pos = pos + 1
			if pos >= 50:
				print 'Headers won\'t end in Radiance image...'
				raise SyntaxError, 'Headers won\'t end in Radiance image...'
			line = file.readline()
		lines.append(line)
		return lines

	def _parseheaders(self, headerlines):
		data = {'head':None,
				'commands':[],
				'format':None,
				'view':None,
				'exposure':None,
				'software':None,
				'unknown':[]}
		for line in headerlines:
			line = string.rstrip(line)
			if   line     in ['', '\n']:  pass
			elif line[:2] == '#?':        data['head']     = line
			elif line[:9] == 'SOFTWARE=': data['software'] = line[9:]
			elif line[:5] == 'VIEW=':     data['view']     = line[5:]
			elif line[:7] == 'FORMAT=':   data['format']   = line[7:]
			elif line[:9] == 'EXPOSURE=': data['exposure'] = line[9:]
			else:
				data['commands'].append(line)
		return data

	def _readimgsize(self, file):
		line = file.readline(32) # we don't really expect more than 18 chars.
		strs = string.split(line, None, 4)[:4]
		if len(strs) < 4:
			print 'Incomplete resolution info in Radiance image'
			raise SyntaxError, 'Incomplete resolution info in Radiance image'
		if strs[0][0] not in ['-', '+']:
			print 'Y resolution sign missing in Radiance image'
			raise SyntaxError, 'Y resolution sign missing in Radiance image'
		if strs[0][1] != 'Y':
			print 'Y resolution code missing in Radiance image'
			raise SyntaxError, 'Y resolution code missing in Radiance image'
		try: yval = string.atoi(strs[1])
		except ValueError:
			print 'Y resolution not an integer in Radiance image'
			raise SyntaxError, 'Y resolution not an integer in Radiance image'
		if strs[0][0] == '-': yval = - yval
		if strs[2][0] not in ['-', '+']:
			print 'X resolution sign missing in Radiance image'
			raise SyntaxError, 'X resolution sign missing in Radiance image'
		if strs[2][1] != 'X':
			print 'X resolution code missing in Radiance image'
			raise SyntaxError, 'X resolution code missing in Radiance image'
		try: xval = string.atoi(strs[3])
		except ValueError:
			print 'X resolution not an integer in Radiance image'
			raise SyntaxError, 'X resolution not an integer in Radiance image'
		if strs[2][0] == '-': xval = - xval
		# this should be the position after the '\n'
		self.start = file.tell()
		return (xval, yval)


Image.register_open(_radiance_format, RadianceImageFile, None)#_accept)
Image.register_save(_radiance_format, _save)
Image.register_extension(_radiance_format, ".pic")
Image.register_extension(_radiance_format, ".unf")
Image.register_mime(_radiance_format, 'image/x-radiance')
Image.register_mime(_radiance_format, 'image/x-32-bit_rle_rgbe')
